@@ -8,11 +8,8 @@ import warnings
 
 # Third-party
 from astropy.coordinates import (EarthLocation, SkyCoord, AltAz, get_sun,
-                                 get_moon, Angle, Latitude, Longitude,
-                                 UnitSphericalRepresentation)
+                                 get_moon, Angle, Longitude)
 from astropy.extern.six import string_types
-from astropy.utils import isiterable
-from astropy.utils.compat.numpy import broadcast_to
 import astropy.units as u
 from astropy.time import Time
 import numpy as np
@@ -21,7 +18,7 @@ import pytz
 # Package
 from .exceptions import TargetNeverUpWarning, TargetAlwaysUpWarning
 from .moon import moon_illumination, moon_phase_angle
-from .target import get_skycoord
+from .target import get_skycoord, SpecialObjectFlag, SunFlag, MoonFlag
 
 
 __all__ = ["Observer", "MAGIC_TIME"]
@@ -215,8 +212,8 @@ class Observer(object):
                     formatted_loc = ["{} {}".format(i.value, i.unit)
                                      for i in value.to_geodetic()]
                     attributes_strings.append(
-                        "{} (lon, lat, el)=({})".format(name,
-                                                        ", ".join(formatted_loc)))
+                        "{} (lon, lat, el)=({})".format(
+                            name, ", ".join(formatted_loc)))
                 else:
                     if name != 'name':
                         value = repr(value)
@@ -358,7 +355,7 @@ class Observer(object):
                 return False
         return True
 
-    def _preprocess_inputs(self, time, target=None, grid=True):
+    def _preprocess_inputs(self, time, target=None, grid_times_targets=False):
         """
         Preprocess time and target inputs
 
@@ -373,11 +370,11 @@ class Observer(object):
         target : `~astroplan.FixedTarget`, `~astropy.coordinates.SkyCoord`, or list
             The target(s) to use in the calculation.
 
-        grid: bool
-            If True, and the time and target objects cannot be broadcast,
-            the target object will have extra dimensions packed onto the end,
+        grid_times_targets: bool
+            If True, the target object will have extra dimensions packed onto the end,
             so that calculations with M targets and N times will return an (M, N)
-            shaped result. Useful for grid searches for rise/set times etc.
+            shaped result. Otherwise, we rely on broadcasting the shapes together
+            using standard numpy rules. Useful for grid searches for rise/set times etc.
         """
         # make sure we have a non-scalar time
         if not isinstance(time, Time):
@@ -388,23 +385,21 @@ class Observer(object):
 
         # convert any kind of target argument to non-scalar SkyCoord
         target = get_skycoord(target)
-
-        if grid:
-            # now we broadcast the targets array so that the first index
-            # iterates over targets, any other indices over times
-            if not target.isscalar:
-                if time.isscalar:
-                    target = target[:, np.newaxis]
+        if grid_times_targets:
+            if target.isscalar:
+                # ensure we have a (1, 1) shape coord
+                target = SkyCoord(np.tile(target, 1))[:, np.newaxis]
+            else:
                 while target.ndim <= time.ndim:
                     target = target[:, np.newaxis]
-        if not self._is_broadcastable(target.shape, time.shape):
-            raise ValueError(
-                'Time and Target arguments cannot be broadcast against each other with shapes {} and {}'.format(
-                    time.shape, target.shape
-                ))
+
+        elif not self._is_broadcastable(target.shape, time.shape):
+            raise ValueError('Time and Target arguments cannot be broadcast '
+                             'against each other with shapes {} and {}'
+                             .format(time.shape, target.shape))
         return time, target
 
-    def altaz(self, time, target=None, obswl=None, grid=True):
+    def altaz(self, time, target=None, obswl=None, grid_times_targets=False):
         """
         Get an `~astropy.coordinates.AltAz` frame or coordinate.
 
@@ -420,18 +415,19 @@ class Observer(object):
             initializer, so it can be anything that `~astropy.time.Time` will
             accept (including a `~astropy.time.Time` object)
 
-        target : `~astroplan.FixedTarget`, `~astropy.coordinates.SkyCoord`, or list; defaults to `None` (optional)
-            Celestial object(s) of interest. If ``target`` is `None`, return the
-            `~astropy.coordinates.AltAz` frame without coordinates.
+        target : `~astroplan.FixedTarget`, `~astropy.coordinates.SkyCoord`, or list (optional)
+            Celestial object(s) of interest. If ``target`` is `None`, returns
+            the `~astropy.coordinates.AltAz` frame without coordinates.
 
         obswl : `~astropy.units.Quantity` (optional)
             Wavelength of the observation used in the calculation.
 
-        grid: bool
-            If True, and the time and target objects cannot be broadcast,
-            the target object will have extra dimensions packed onto the end,
-            so that calculations with M targets and N times will return an (M, N)
-            shaped result. Useful for grid searches for rise/set times etc.
+        grid_times_targets: bool (optional)
+            If True, the target object will have extra dimensions packed
+            onto the end, so that calculations with M targets and N times
+            will return an (M, N) shaped result. Otherwise, we rely on
+            broadcasting the shapes together using standard numpy
+            rules. Useful for grid searches for rise/set times etc.
 
         Returns
         -------
@@ -463,7 +459,7 @@ class Observer(object):
         >>> target_altaz = apo.altaz(time, target) # doctest: +SKIP
         """
         if target is not None:
-            time, target = self._preprocess_inputs(time, target, grid)
+            time, target = self._preprocess_inputs(time, target, grid_times_targets)
 
         altaz_frame = AltAz(location=self.location, obstime=time,
                             pressure=self.pressure, obswl=obswl,
@@ -475,7 +471,7 @@ class Observer(object):
         else:
             return target.transform_to(altaz_frame)
 
-    def parallactic_angle(self, time, target):
+    def parallactic_angle(self, time, target, grid_times_targets=False):
         """
         Calculate the parallactic angle.
 
@@ -487,6 +483,12 @@ class Observer(object):
         target : `~astroplan.FixedTarget` or `~astropy.coordinates.SkyCoord` or list
             Target celestial object(s).
 
+        grid_times_targets: bool
+            If True, the target object will have extra dimensions packed onto the end,
+            so that calculations with M targets and N times will return an (M, N)
+            shaped result. Otherwise, we rely on broadcasting the shapes together
+            using standard numpy rules.
+
         Returns
         -------
         `~astropy.coordinates.Angle`
@@ -501,13 +503,13 @@ class Observer(object):
         .. [1] https://en.wikipedia.org/wiki/Parallactic_angle
 
         """
-        time, coordinate = self._preprocess_inputs(time, target)
+        time, coordinate = self._preprocess_inputs(time, target, grid_times_targets)
 
         # Eqn (14.1) of Meeus' Astronomical Algorithms
-        LST = time.sidereal_time('mean', longitude=self.location.longitude)
+        LST = time.sidereal_time('mean', longitude=self.location.lon)
         H = (LST - coordinate.ra).radian
         q = np.arctan(np.sin(H) /
-                      (np.tan(self.location.latitude.radian) *
+                      (np.tan(self.location.lat.radian) *
                        np.cos(coordinate.dec.radian) -
                        np.sin(coordinate.dec.radian)*np.cos(H)))*u.rad
 
@@ -557,7 +559,7 @@ class Observer(object):
             ntargets = alt.shape[1]
             ngrid = alt.shape[0]
             unit = alt.unit
-            alt = broadcast_to(alt, (ntargets, ngrid, ntargets)).T
+            alt = np.broadcast_to(alt, (ntargets, ngrid, ntargets)).T
             alt = alt*unit
             extra_dimension_added = True
             if t.shape[1] == 1:
@@ -594,9 +596,8 @@ class Observer(object):
 
         before_indices = np.array(np.nonzero(condition))
         # we want to add an vector like (0, 1, ...) to get after indices
-        array_to_add = np.zeros(before_indices.shape[0])[:, np.newaxis].astype(int)
-        array_to_add[1] = 1
-        after_indices = before_indices + array_to_add
+        after_indices = before_indices.copy()
+        after_indices[1, :] += 1
 
         al1 = alt[tuple(before_indices)]
         al2 = alt[tuple(after_indices)]
@@ -657,7 +658,7 @@ class Observer(object):
         crossing_jd[np.isnan(crossing_jd)] = u.d*MAGIC_TIME.jd
         return np.squeeze(Time(crossing_jd, format='jd'))
 
-    def _altitude_trig(self, LST, target):
+    def _altitude_trig(self, LST, target, grid_times_targets=False):
         """
         Calculate the altitude of ``target`` at local sidereal times ``LST``.
 
@@ -672,20 +673,27 @@ class Observer(object):
         target : {`~astropy.coordinates.SkyCoord`, `FixedTarget`} or similar
             Target celestial object's coordinates.
 
+        grid_times_targets: bool
+            If True, the target object will have extra dimensions packed onto the end,
+            so that calculations with M targets and N times will return an (M, N)
+            shaped result. Otherwise, we rely on broadcasting the shapes together
+            using standard numpy rules. Useful for grid searches for rise/set times etc.
+
         Returns
         -------
         alt : `~astropy.unit.Quantity`
             Array of altitudes
         """
-        LST, target = self._preprocess_inputs(LST, target)
-        alt = np.arcsin(np.sin(self.location.latitude.radian) *
+        LST, target = self._preprocess_inputs(LST, target, grid_times_targets)
+        alt = np.arcsin(np.sin(self.location.lat.radian) *
                         np.sin(target.dec) +
-                        np.cos(self.location.latitude.radian) *
+                        np.cos(self.location.lat.radian) *
                         np.cos(target.dec) *
                         np.cos(LST.radian - target.ra.radian))
         return alt
 
-    def _calc_riseset(self, time, target, prev_next, rise_set, horizon, N=150):
+    def _calc_riseset(self, time, target, prev_next, rise_set, horizon,
+                      N=150, grid_times_targets=False):
         """
         Time at next rise/set of ``target``.
 
@@ -716,6 +724,12 @@ class Observer(object):
             Number of altitudes to compute when searching for
             rise or set.
 
+        grid_times_targets: bool
+            If True, the target object will have extra dimensions packed onto the end,
+            so that calculations with M targets and N times will return an (M, N)
+            shaped result. Otherwise, we rely on broadcasting the shapes together
+            using standard numpy rules.
+
         Returns
         -------
         ret1 : `~astropy.time.Time`
@@ -725,11 +739,26 @@ class Observer(object):
             time = Time(time)
 
         if prev_next == 'next':
-            times = _generate_24hr_grid(time, 0, 1, N)
+            start = 0
+            end = (1 + (target.approx_sidereal_drift.to(u.day).value
+                        if hasattr(target, 'approx_sidereal_drift') else 0))
         else:
-            times = _generate_24hr_grid(time, -1, 0, N)
+            start = (-1 - (target.approx_sidereal_drift.to(u.day).value
+                           if hasattr(target, 'approx_sidereal_drift') else 0))
+            end = 0
+
+        times = _generate_24hr_grid(time, start, end, N)
+
+        if target is MoonFlag:
+            altaz = self.altaz(times, get_moon(times, location=self.location),
+                               grid_times_targets=grid_times_targets)
+        elif target is SunFlag:
+            altaz = self.altaz(times, get_sun(times),
+                               grid_times_targets=grid_times_targets)
+        else:
+            altaz = self.altaz(times, target,
+                               grid_times_targets=grid_times_targets)
 
-        altaz = self.altaz(times, target, grid=True)
         altitudes = altaz.alt
 
         al1, al2, jd1, jd2 = self._horiz_cross(times, altitudes, rise_set,
@@ -737,7 +766,8 @@ class Observer(object):
         return self._two_point_interp(jd1, jd2, al1, al2,
                                       horizon=horizon)
 
-    def _calc_transit(self, time, target, prev_next, antitransit=False, N=150):
+    def _calc_transit(self, time, target, prev_next, antitransit=False,
+                      N=150, grid_times_targets=False):
         """
         Time at next transit of the meridian of `target`.
 
@@ -764,6 +794,12 @@ class Observer(object):
             Number of altitudes to compute when searching for
             rise or set.
 
+        grid_times_targets: bool
+            If True, the target object will have extra dimensions packed onto the end,
+            so that calculations with M targets and N times will return an (M, N)
+            shaped result. Otherwise, we rely on broadcasting the shapes together
+            using standard numpy rules.
+
         Returns
         -------
         ret1 : `~astropy.time.Time`
@@ -785,7 +821,7 @@ class Observer(object):
         else:
             rise_set = 'setting'
 
-        altaz = self.altaz(times, target, grid=True)
+        altaz = self.altaz(times, target, grid_times_targets=grid_times_targets)
         altitudes = altaz.alt
         if altitudes.ndim > 2:
             # shape is (M, N, ...) where M is targets and N is grid
@@ -814,12 +850,17 @@ class Observer(object):
         horizon = args_dict.pop('horizon', None)
         rise_set = args_dict.pop('rise_set', None)
         antitransit = args_dict.pop('antitransit', None)
+        grid_times_targets = args_dict.pop('grid_times_targets', False)
 
         # Assemble arguments for function, depending on the function.
         if function == self._calc_riseset:
-            args = lambda w: (time, target, w, rise_set, horizon)
+            def event_function(w):
+                return function(time, target, w, rise_set, horizon,
+                                grid_times_targets=grid_times_targets)
         elif function == self._calc_transit:
-            args = lambda w: (time, target, w, antitransit)
+            def event_function(w):
+                return function(time, target, w, antitransit=antitransit,
+                                grid_times_targets=grid_times_targets)
         else:
             raise ValueError('Function {} not supported in '
                              '_determine_which_event.'.format(function))
@@ -828,26 +869,26 @@ class Observer(object):
             time = Time(time)
 
         if which == 'next' or which == 'nearest':
-            next_event = function(*args('next'))
+            next_event = event_function('next')
             if which == 'next':
                 return next_event
 
         if which == 'previous' or which == 'nearest':
-            previous_event = function(*args('previous'))
+            previous_event = event_function('previous')
             if which == 'previous':
                 return previous_event
 
         if which == 'nearest':
             mask = abs(time - previous_event) < abs(time - next_event)
             return Time(np.where(mask, previous_event.utc.jd,
-                        next_event.utc.jd), format='jd')
-
+                                 next_event.utc.jd), format='jd')
 
         raise ValueError('"which" kwarg must be "next", "previous" or '
                          '"nearest".')
 
     @u.quantity_input(horizon=u.deg)
-    def target_rise_time(self, time, target, which='nearest', horizon=0*u.degree):
+    def target_rise_time(self, time, target, which='nearest', horizon=0*u.degree,
+                         grid_times_targets=False):
         """
         Calculate rise time.
 
@@ -864,7 +905,7 @@ class Observer(object):
             `~astropy.time.Time` will accept (including a `~astropy.time.Time`
             object)
 
-        target : coordinate object (i.e. `~astropy.coordinates.SkyCoord`, `~astroplan.FixedTarget`) or list
+        target : `~astropy.coordinates.SkyCoord`, `~astroplan.FixedTarget`, or list
             Target celestial object(s)
 
         which : {'next', 'previous', 'nearest'}
@@ -876,6 +917,12 @@ class Observer(object):
             for calculating rise/set times (i.e.,
             -6 deg horizon = civil twilight, etc.)
 
+        grid_times_targets: bool
+            If True, the target object will have extra dimensions packed
+            onto the end, so that calculations with M targets and N times
+            will return an (M, N) shaped result. Otherwise, we rely on
+            broadcasting the shapes together using standard numpy rules.
+
         Returns
         -------
         `~astropy.time.Time`
@@ -890,17 +937,19 @@ class Observer(object):
         >>> time = Time("2001-02-03 04:05:06")
         >>> target = FixedTarget.from_name("Rigel")
         >>> keck = Observer.at_site("Keck")
-        >>> rigel_rise_time = keck.target_rise_time(time, target, which="next")
-        >>> print("ISO: {0.iso}, JD: {0.jd}".format(rigel_rise_time)) # doctest: +FLOAT_CMP
+        >>> rigel_rise_time = keck.target_rise_time(time, target, which="next") # doctest: +SKIP
+        >>> print("ISO: {0.iso}, JD: {0.jd}".format(rigel_rise_time)) # doctest: +SKIP
         ISO: 2001-02-04 00:51:23.330, JD: 2451944.53569
         """
         return self._determine_which_event(self._calc_riseset,
                                            dict(time=time, target=target,
                                                 which=which, rise_set='rising',
-                                                horizon=horizon))
+                                                horizon=horizon,
+                                                grid_times_targets=grid_times_targets))
 
     @u.quantity_input(horizon=u.deg)
-    def target_set_time(self, time, target, which='nearest', horizon=0*u.degree):
+    def target_set_time(self, time, target, which='nearest', horizon=0*u.degree,
+                        grid_times_targets=False):
         """
         Calculate set time.
 
@@ -916,7 +965,7 @@ class Observer(object):
             `~astropy.time.Time` will accept (including a `~astropy.time.Time`
             object)
 
-        target : coordinate object (i.e. `~astropy.coordinates.SkyCoord`, `~astroplan.FixedTarget`) or list
+        target : `~astropy.coordinates.SkyCoord`, `~astroplan.FixedTarget`, or list
             Target celestial object(s)
 
         which : {'next', 'previous', 'nearest'}
@@ -928,6 +977,12 @@ class Observer(object):
             for calculating rise/set times (i.e.,
             -6 deg horizon = civil twilight, etc.)
 
+        grid_times_targets: bool
+            If True, the target object will have extra dimensions packed
+            onto the end, so that calculations with M targets and N times
+            will return an (M, N) shaped result. Otherwise, we rely on
+            broadcasting the shapes together using standard numpy rules.
+
         Returns
         -------
         `~astropy.time.Time`
@@ -942,16 +997,17 @@ class Observer(object):
         >>> time = Time("2001-02-03 04:05:06")
         >>> target = FixedTarget.from_name("Rigel")
         >>> keck = Observer.at_site("Keck")
-        >>> rigel_set_time = keck.target_set_time(time, target, which="next")
-        >>> print("ISO: {0.iso}, JD: {0.jd}".format(rigel_set_time)) # doctest: +FLOAT_CMP
+        >>> rigel_set_time = keck.target_set_time(time, target, which="next") # doctest: +SKIP
+        >>> print("ISO: {0.iso}, JD: {0.jd}".format(rigel_set_time)) # doctest: +SKIP
         ISO: 2001-02-03 12:29:34.768, JD: 2451944.02054
         """
         return self._determine_which_event(self._calc_riseset,
                                            dict(time=time, target=target,
                                                 which=which, rise_set='setting',
-                                                horizon=horizon))
+                                                horizon=horizon,
+                                                grid_times_targets=grid_times_targets))
 
-    def target_meridian_transit_time(self, time, target, which='nearest'):
+    def target_meridian_transit_time(self, time, target, which='nearest', grid_times_targets=False):
         """
         Calculate time at the transit of the meridian.
 
@@ -966,13 +1022,19 @@ class Observer(object):
             `~astropy.time.Time` will accept (including a `~astropy.time.Time`
             object)
 
-        target : coordinate object (i.e. `~astropy.coordinates.SkyCoord`, `~astroplan.FixedTarget`) or list
+        target : `~astropy.coordinates.SkyCoord`, `~astroplan.FixedTarget`, or list
             Target celestial object(s)
 
         which : {'next', 'previous', 'nearest'}
             Choose which sunrise relative to the present ``time`` would you
             like to calculate
 
+        grid_times_targets: bool
+            If True, the target object will have extra dimensions packed
+            onto the end, so that calculations with M targets and N times
+            will return an (M, N) shaped result. Otherwise, we rely on
+            broadcasting the shapes together using standard numpy rules.
+
         Returns
         -------
         `~astropy.time.Time`
@@ -988,16 +1050,18 @@ class Observer(object):
         >>> target = FixedTarget.from_name("Rigel")
         >>> keck = Observer.at_site("Keck")
         >>> rigel_transit_time = keck.target_meridian_transit_time(time, target,
-        ...                                                        which="next")
-        >>> print("ISO: {0.iso}, JD: {0.jd}".format(rigel_transit_time)) # doctest: +FLOAT_CMP
+        ...                                                        which="next") # doctest: +SKIP
+        >>> print("ISO: {0.iso}, JD: {0.jd}".format(rigel_transit_time)) # doctest: +SKIP
         ISO: 2001-02-03 06:42:26.863, JD: 2451943.77948
         """
         return self._determine_which_event(self._calc_transit,
                                            dict(time=time, target=target,
                                                 which=which,
-                                                rise_set='setting'))
+                                                rise_set='setting',
+                                                grid_times_targets=grid_times_targets))
 
-    def target_meridian_antitransit_time(self, time, target, which='nearest'):
+    def target_meridian_antitransit_time(self, time, target, which='nearest',
+                                         grid_times_targets=False):
         """
         Calculate time at the antitransit of the meridian.
 
@@ -1012,13 +1076,19 @@ class Observer(object):
             `~astropy.time.Time` will accept (including a `~astropy.time.Time`
             object).
 
-        target : coordinate object (i.e. `~astropy.coordinates.SkyCoord`, `~astroplan.FixedTarget`) or list
+        target : `~astropy.coordinates.SkyCoord`, `~astroplan.FixedTarget`, or list
             Target celestial object(s)
 
         which : {'next', 'previous', 'nearest'}
             Choose which sunrise relative to the present ``time`` would you
             like to calculate
 
+        grid_times_targets : bool
+            If True, the target object will have extra dimensions packed onto the end,
+            so that calculations with M targets and N times will return an (M, N)
+            shaped result. Otherwise, we rely on broadcasting the shapes together
+            using standard numpy rules.
+
         Returns
         -------
         `~astropy.time.Time`
@@ -1033,16 +1103,17 @@ class Observer(object):
         >>> time = Time("2001-02-03 04:05:06")
         >>> target = FixedTarget.from_name("Rigel")
         >>> keck = Observer.at_site("Keck")
-        >>> rigel_antitransit_time = keck.target_meridian_antitransit_time(time, target,
-        ...                                                                which="next")
-        >>> print("ISO: {0.iso}, JD: {0.jd}".format(rigel_antitransit_time)) # doctest: +FLOAT_CMP
+        >>> rigel_antitransit_time = keck.target_meridian_antitransit_time(
+        ...     time, target, which="next") # doctest: +SKIP
+        >>> print("ISO: {0.iso}, JD: {0.jd}".format(rigel_antitransit_time)) # doctest: +SKIP
         ISO: 2001-02-03 18:40:29.761, JD: 2451944.27812
 
         """
         return self._determine_which_event(self._calc_transit,
                                            dict(time=time, target=target,
                                                 which=which, antitransit=True,
-                                                rise_set='setting'))
+                                                rise_set='setting',
+                                                grid_times_targets=grid_times_targets))
 
     @u.quantity_input(horizon=u.deg)
     def sun_rise_time(self, time, which='nearest', horizon=0*u.degree):
@@ -1083,7 +1154,7 @@ class Observer(object):
         >>> from astropy.time import Time
         >>> apo = Observer.at_site("APO")
         >>> time = Time('2001-02-03 04:05:06')
-        >>> sun_rise = apo.sun_rise_time(time, which="previous")
+        >>> sun_rise = apo.sun_rise_time(time, which="previous") # doctest: +SKIP
         >>> print("ISO: {0.iso}, JD: {0.jd}".format(sun_rise)) # doctest: +SKIP
         ISO: 2001-02-02 14:02:50.554, JD: 2451943.08531
         """
@@ -1128,7 +1199,7 @@ class Observer(object):
         >>> from astropy.time import Time
         >>> apo = Observer.at_site("APO")
         >>> time = Time('2001-02-03 04:05:06')
-        >>> sun_set = apo.sun_set_time(time, which="next")
+        >>> sun_set = apo.sun_set_time(time, which="next") # doctest: +SKIP
         >>> print("ISO: {0.iso}, JD: {0.jd}".format(sun_set)) # doctest: +SKIP
         ISO: 2001-02-04 00:35:42.102, JD: 2451944.52479
         """
@@ -1322,41 +1393,61 @@ class Observer(object):
 
     # Moon-related methods.
 
-    def moon_rise_time(self, time, **kwargs):
+    def moon_rise_time(self, time, which='nearest', horizon=0*u.deg):
         """
-        Returns the local moonrise time.
+        Returns the local moon rise time.
 
-        The default moonrise returned is the next one to occur.
+        Compute time of the next/previous/nearest moon rise, where
+        moon rise is defined as the time when the moon transitions from
+        altitudes below ``horizon`` to above ``horizon``.
 
         Parameters
         ----------
         time : `~astropy.time.Time` or other (see below)
-
-        Keywords : str, optional
-            previous
-            next
-        """
-        raise NotImplementedError()
-
-    def moon_set_time(self, time, **kwargs):
-        """
-        Returns the local moonset time.
-
-        The default moonset returned is the next one to occur.
-
-        Parameters
-        ----------
-        time : `~astropy.time.Time` or other (see below)
-            This will be passed in as the first argument to
+            Time of observation. This will be passed in as the first argument to
             the `~astropy.time.Time` initializer, so it can be anything that
             `~astropy.time.Time` will accept (including a `~astropy.time.Time`
             object).
 
-        Keywords : str, optional
-            previous
-            next
+        which : {'next', 'previous', 'nearest'}
+            Choose which moon rise relative to the present ``time`` would you
+            like to calculate.
+
+        horizon : `~astropy.units.Quantity` (optional), default = zero degrees
+            Degrees above/below actual horizon to use
+            for calculating rise/set times (i.e.,
+            -6 deg horizon = civil twilight, etc.)
+
         """
-        raise NotImplementedError()
+        return self.target_rise_time(time, MoonFlag, which, horizon)
+
+    def moon_set_time(self, time, which='nearest', horizon=0*u.deg):
+        """
+        Returns the local moon set time.
+
+        Compute time of the next/previous/nearest moon set, where
+        moon set is defined as the time when the moon transitions from
+        altitudes below ``horizon`` to above ``horizon``.
+
+        Parameters
+        ----------
+        time : `~astropy.time.Time` or other (see below)
+            Time of observation. This will be passed in as the first argument to
+            the `~astropy.time.Time` initializer, so it can be anything that
+            `~astropy.time.Time` will accept (including a `~astropy.time.Time`
+            object).
+
+        which : {'next', 'previous', 'nearest'}
+            Choose which moon set relative to the present ``time`` would you
+            like to calculate.
+
+        horizon : `~astropy.units.Quantity` (optional), default = zero degrees
+            Degrees above/below actual horizon to use
+            for calculating set/set times (i.e.,
+            -6 deg horizon = civil twilight, etc.)
+
+        """
+        return self.target_set_time(time, MoonFlag, which, horizon)
 
     def moon_illumination(self, time):
         """
@@ -1469,10 +1560,40 @@ class Observer(object):
             time = Time(time)
 
         moon = get_moon(time, location=self.location, ephemeris=ephemeris)
-        return self.altaz(time, moon, grid=False)
+        return self.altaz(time, moon)
+
+    def sun_altaz(self, time):
+        """
+        Returns the position of the Sun in alt/az.
+
+        Parameters
+        ----------
+        time : `~astropy.time.Time` or other (see below)
+            This will be passed in as the first argument to
+            the `~astropy.time.Time` initializer, so it can be anything that
+            `~astropy.time.Time` will accept (including a `~astropy.time.Time`
+            object).
+
+        ephemeris : str, optional
+            Ephemeris to use.  If not given, use the one set with
+            ``astropy.coordinates.solar_system_ephemeris.set`` (which is
+            set to 'builtin' by default).
+
+
+        Returns
+        -------
+        altaz : `~astropy.coordinates.SkyCoord`
+            Position of the moon transformed to altitude and azimuth
+        """
+        if not isinstance(time, Time):
+            time = Time(time)
+
+        sun = get_sun(time)
+        return self.altaz(time, sun)
 
     @u.quantity_input(horizon=u.deg)
-    def target_is_up(self, time, target, horizon=0*u.degree, return_altaz=False):
+    def target_is_up(self, time, target, horizon=0*u.degree,
+                     return_altaz=False, grid_times_targets=False):
         """
         Is ``target`` above ``horizon`` at this ``time``?
 
@@ -1484,7 +1605,7 @@ class Observer(object):
             `~astropy.time.Time` will accept (including a `~astropy.time.Time`
             object)
 
-        target : coordinate object (i.e. `~astropy.coordinates.SkyCoord`, `~astroplan.FixedTarget`) or list
+        target : `~astropy.coordinates.SkyCoord`, `~astroplan.FixedTarget`, or list
             Target celestial object(s)
 
         horizon : `~astropy.units.Quantity` (optional), default = zero degrees
@@ -1495,6 +1616,12 @@ class Observer(object):
         return_altaz : bool (optional)
             Also return the '~astropy.coordinates.AltAz' coordinate.
 
+        grid_times_targets: bool
+            If True, the target object will have extra dimensions packed
+            onto the end, so that calculations with M targets and N times
+            will return an (M, N) shaped result. Otherwise, we rely on
+            broadcasting the shapes together using standard numpy rules.
+
         Returns
         -------
         observable : boolean or np.ndarray(bool)
@@ -1511,22 +1638,19 @@ class Observer(object):
         >>> time = Time("2015-08-29 18:35")
         >>> aldebaran = FixedTarget.from_name("Aldebaran")
         >>> vega = FixedTarget.from_name("Vega")
-        >>> apo.target_is_up(time, aldebaran)
+        >>> apo.target_is_up(time, aldebaran) # doctest: +SKIP
         True
-        >>> apo.target_is_up(time, [aldebaran, vega])
-        [True, False]
+        >>> apo.target_is_up(time, [aldebaran, vega]) # doctest: +SKIP
+        array([ True, False], dtype=bool)
         """
         if not isinstance(time, Time):
             time = Time(time)
 
-        altaz = self.altaz(time, target)
+        altaz = self.altaz(time, target, grid_times_targets=grid_times_targets)
         observable = altaz.alt > horizon
+
         if altaz.isscalar:
             observable = bool(observable)
-        else:
-            # TODO: simply return observable if we move to
-            # a fully broadcasted API
-            observable = [value for value in observable.flat]
 
         if not return_altaz:
             return observable
@@ -1575,12 +1699,11 @@ class Observer(object):
             time = Time(time)
 
         solar_altitude = self.altaz(time, target=get_sun(time), obswl=obswl).alt
+
         if solar_altitude.isscalar:
             return bool(solar_altitude < horizon)
         else:
-            # TODO: simply return solar_altitude < horizon if we move to
-            # a fully broadcasted API
-            return [val for val in (solar_altitude < horizon).flat]
+            return solar_altitude < horizon
 
     def local_sidereal_time(self, time, kind='apparent', model=None):
         """
@@ -1613,10 +1736,10 @@ class Observer(object):
         if not isinstance(time, Time):
             time = Time(time)
 
-        return time.sidereal_time(kind, longitude=self.location.longitude,
+        return time.sidereal_time(kind, longitude=self.location.lon,
                                   model=model)
 
-    def target_hour_angle(self, time, target):
+    def target_hour_angle(self, time, target, grid_times_targets=False):
         """
         Calculate the local hour angle of ``target`` at ``time``.
 
@@ -1628,15 +1751,21 @@ class Observer(object):
             `~astropy.time.Time` will accept (including a `~astropy.time.Time`
             object)
 
-        target : coordinate object (i.e. `~astropy.coordinates.SkyCoord`, `~astroplan.FixedTarget`) or list
+        target : `~astropy.coordinates.SkyCoord`, `~astroplan.FixedTarget`, or list
             Target celestial object(s)
 
+        grid_times_targets: bool
+            If True, the target object will have extra dimensions packed
+            onto the end, so that calculations with M targets and N times
+            will return an (M, N) shaped result. Otherwise, we rely on
+            broadcasting the shapes together using standard numpy rules.
+
         Returns
         -------
         hour_angle : `~astropy.coordinates.Angle`
             The hour angle(s) of the target(s) at ``time``
         """
-        time, target = self._preprocess_inputs(time, target)
+        time, target = self._preprocess_inputs(time, target, grid_times_targets)
         return Longitude(self.local_sidereal_time(time) - target.ra)
 
     @u.quantity_input(horizon=u.degree)
@@ -1668,14 +1797,11 @@ class Observer(object):
         current_time = Time.now() if time is None else time
         night_mask = self.is_night(current_time, horizon=horizon, obswl=obswl)
         sun_set_time = self.sun_set_time(current_time, which='next', horizon=horizon)
-        # workaround for NPY <= 1.8, otherwise np.where works even in scalar case
-        if current_time.isscalar:
-            start_time = current_time if night_mask else sun_set_time
-        else:
-            start_time = np.where(night_mask, current_time, sun_set_time)
-            # np.where gives us a list of start Times - convert to Time object
-            if not isinstance(start_time, Time):
-                start_time = Time(start_time)
+
+        start_time = np.where(night_mask, current_time, sun_set_time)
+        # np.where gives us a list of start Times - convert to Time object
+        if not isinstance(start_time, Time):
+            start_time = Time(start_time)
         end_time = self.sun_rise_time(start_time, which='next', horizon=horizon)
 
         return start_time, end_time
