@@ -31,15 +31,14 @@ def plot_sky(target, observer, time, ax=None, style_kwargs=None,
     (e.g. ``Time(['2000-1-1 20:00:00', '2000-1-1 20:30:00'])``), target's
     position will be shown at each of these times.
 
-    For examples with plots, visit the astroplan Read the Docs
-    documentation [1]_.
+    For examples with plots, visit the documentation of :ref:`plots_sky_charts`.
 
     Parameters
     ----------
     target : `~astroplan.FixedTarget`
         The celestial body of interest.
 
-    observer: `~astroplan.Observer`
+    observer : `~astroplan.Observer`
         The person, telescope, observatory, etc. doing the observing.
 
     time : `~astropy.time.Time`
@@ -109,9 +108,6 @@ def plot_sky(target, observer, time, ax=None, style_kwargs=None,
         makes it seem as if N/E/S/W are being decoupled from the definition
         of azimuth (North from az = 0 deg., East from az = 90 deg., etc.).
 
-    References
-    ----------
-    .. [1] astroplan plotting tutorial: https://astroplan.readthedocs.io/en/latest/tutorials/plots.html#sky-charts
     """
     # Import matplotlib, set style sheet
     if style_sheet is not None:
@@ -196,7 +192,7 @@ def plot_sky(target, observer, time, ax=None, style_kwargs=None,
         '30' + degree_sign,
         '',
         '0' + degree_sign + ' Alt.',
-        ]
+    ]
 
     theta_labels = []
     for chunk in range(0, 7):
@@ -228,6 +224,7 @@ def plot_sky(target, observer, time, ax=None, style_kwargs=None,
 
     return ax
 
+
 @u.quantity_input(delta=u.hour)
 def plot_sky_24hr(target, observer, time, delta=1*u.hour, ax=None,
                   style_kwargs=None, north_to_east_ccw=True, grid=True,
