@@ -49,7 +49,8 @@ def _has_twin(ax):
 
 def plot_airmass(targets, observer, time, ax=None, style_kwargs=None,
                  style_sheet=None, brightness_shading=False,
-                 altitude_yaxis=False):
+                 altitude_yaxis=False, min_airmass=1.0, min_region=None,
+                 max_airmass=3.0, max_region=None):
     r"""
     Plots airmass as a function of time for a given target.
 
@@ -66,8 +67,8 @@ def plot_airmass(targets, observer, time, ax=None, style_kwargs=None,
     object. For instance, ``Time(['2000-1-1 23:00:00', '2000-1-1
     23:30:00'])`` will result in a plot with only two airmass measurements.
 
-    For examples with plots, visit the astroplan Read the Docs
-    documentation [1]_.
+    For examples with plots, visit the documentation of
+    :ref:`plots_time_dependent`.
 
     Parameters
     ----------
@@ -107,6 +108,20 @@ def plot_airmass(targets, observer, time, ax=None, style_kwargs=None,
         Add alternative y-axis on the right side of the figure with target
         altitude. Default is `False`.
 
+    min_airmass : float
+        Lower limit of y-axis airmass range in the plot. Default is ``1.0``.
+
+    max_airmass : float
+        Upper limit of y-axis airmass range in the plot. Default is ``3.0``.
+
+    min_region : float
+        If set, defines an interval between ``min_airmass`` and ``min_region``
+        that will be shaded. Default is `None`.
+
+    max_region : float
+        If set, defines an interval between ``max_airmass`` and ``max_region``
+        that will be shaded. Default is `None`.
+
     Returns
     -------
     ax : `~matplotlib.axes.Axes`
@@ -118,10 +133,6 @@ def plot_airmass(targets, observer, time, ax=None, style_kwargs=None,
     If user wishes to change these, use ``ax.<set attribute>`` before drawing
     or saving plot:
 
-    References
-    ----------
-    .. [1] astroplan plotting tutorial: https://astroplan.readthedocs.io/en/latest/tutorials/plots.html#time-dependent-plots
-
     """
     # Import matplotlib, set style sheet
     if style_sheet is not None:
@@ -191,14 +202,24 @@ def plot_airmass(targets, observer, time, ax=None, style_kwargs=None,
 
         twilights.sort(key=operator.itemgetter(0))
         for i, twi in enumerate(twilights[1:], 1):
-            ax.axvspan(twilights[i - 1][0], twilights[i][0], ymin=0, ymax=1, color='grey', alpha=twi[1])
+            ax.axvspan(twilights[i - 1][0], twilights[i][0],
+                       ymin=0, ymax=1, color='grey', alpha=twi[1])
 
     # Invert y-axis and set limits.
-    if ax.get_ylim()[1] > ax.get_ylim()[0]:
+    y_lim = ax.get_ylim()
+    if y_lim[1] > y_lim[0]:
         ax.invert_yaxis()
-    ax.set_ylim([3, 1])
+    ax.set_ylim([max_airmass, min_airmass])
     ax.set_xlim([time[0].plot_date, time[-1].plot_date])
 
+    # Draw lo/hi limit regions, if present
+    ymax, ymin = ax.get_ylim()       # should be (hi_limit, lo_limit)
+
+    if max_region is not None:
+        ax.axhspan(ymax, max_region, facecolor='#F9EB4E', alpha=0.10)
+    if min_region is not None:
+        ax.axhspan(min_region, ymin, facecolor='#F9EB4E', alpha=0.10)
+
     # Set labels.
     ax.set_ylabel("Airmass")
     ax.set_xlabel("Time from {0} [UTC]".format(min(time).datetime.date()))
@@ -242,7 +263,8 @@ def plot_schedule_airmass(schedule, show_night=False):
     blocks = copy.copy(schedule.scheduled_blocks)
     sorted_blocks = sorted(schedule.observing_blocks, key=lambda x: x.priority)
     targets = [block.target for block in sorted_blocks]
-    ts = schedule.start_time + np.linspace(0, (schedule.end_time - schedule.start_time).value, 100) * u.day
+    ts = (schedule.start_time +
+          np.linspace(0, (schedule.end_time - schedule.start_time).value, 100) * u.day)
     targ_to_color = {}
     color_idx = np.linspace(0, 1, len(targets))
     # lighter, bluer colors indicate higher priority
@@ -253,11 +275,15 @@ def plot_schedule_airmass(schedule, show_night=False):
         # I'm pretty sure this overlaps a lot, creating darker bands
         for test_time in ts:
             midnight = schedule.observer.midnight(test_time)
-            previous_sunset = schedule.observer.sun_set_time(midnight, which='previous')
-            next_sunrise = schedule.observer.sun_rise_time(midnight, which='next')
+            previous_sunset = schedule.observer.sun_set_time(
+                midnight, which='previous')
+            next_sunrise = schedule.observer.sun_rise_time(
+                midnight, which='next')
 
-            previous_twilight = schedule.observer.twilight_evening_astronomical(midnight, which='previous')
-            next_twilight = schedule.observer.twilight_morning_astronomical(midnight, which='next')
+            previous_twilight = schedule.observer.twilight_evening_astronomical(
+                midnight, which='previous')
+            next_twilight = schedule.observer.twilight_morning_astronomical(
+                midnight, which='next')
 
             plt.axvspan(previous_sunset.plot_date, next_sunrise.plot_date,
                         facecolor='lightgrey', alpha=0.05)
@@ -293,8 +319,8 @@ def plot_parallactic(target, observer, time, ax=None, style_kwargs=None,
     object. For instance, ``Time(['2000-1-1 23:00:00', '2000-1-1 23:30:00'])``
     will result in a plot with only two parallactic angle measurements.
 
-    For examples with plots, visit the astroplan Read the Docs
-    documentation [1]_.
+    For examples with plots, visit the documentation of
+    :ref:`plots_time_dependent`.
 
     Parameters
     ----------
@@ -329,9 +355,6 @@ def plot_parallactic(target, observer, time, ax=None, style_kwargs=None,
     ax :  `~matplotlib.axes.Axes`
         An ``Axes`` object with added parallactic angle vs. time plot.
 
-    References
-    ----------
-    .. [1] astroplan plotting tutorial: https://astroplan.readthedocs.io/en/latest/tutorials/plots.html#time-dependent-plots
     """
     # Import matplotlib, set style sheet
     if style_sheet is not None:
