@@ -7,7 +7,8 @@ from abc import ABCMeta
 
 # Third-party
 import astropy.units as u
-from astropy.coordinates import SkyCoord, ICRS, UnitSphericalRepresentation
+from astropy.coordinates import (SkyCoord, ICRS, UnitSphericalRepresentation,
+                                 SphericalRepresentation)
 
 __all__ = ["Target", "FixedTarget", "NonFixedTarget"]
 
@@ -87,6 +88,7 @@ class FixedTarget(Target):
     >>> from astroplan import FixedTarget
     >>> sirius = FixedTarget.from_name("Sirius")
     """
+
     def __init__(self, coord, name=None, **kwargs):
         """
         Parameters
@@ -162,6 +164,7 @@ class FixedTarget(Target):
         Mock method to replace `FixedTarget.from_name` in tests without
         internet connection.
         """
+        # The lowercase method will be run on names, so enter keys in lowercase:
         stars = {
             "rigel": {"ra": 78.63446707*u.deg, "dec": -8.20163837*u.deg},
             "sirius": {"ra": 101.28715533*u.deg, "dec": -16.71611586*u.deg},
@@ -170,7 +173,8 @@ class FixedTarget(Target):
             "polaris": {"ra": 37.95456067*u.deg, "dec": 89.26410897*u.deg},
             "deneb": {"ra": 310.35797975*u.deg, "dec": 45.28033881*u.deg},
             "m13": {"ra": 250.423475*u.deg, "dec": 36.4613194*u.deg},
-            "altair": {"ra": 297.6958273*u.deg, "dec": 8.8683212*u.deg}
+            "altair": {"ra": 297.6958273*u.deg, "dec": 8.8683212*u.deg},
+            "hd 209458": {"ra": 330.79*u.deg, "dec": 18.88*u.deg}
         }
 
         if query_name.lower() in stars:
@@ -186,6 +190,7 @@ class NonFixedTarget(Target):
     Placeholder for future function.
     """
 
+
 def get_skycoord(targets):
     """
     Return an `~astropy.coordinates.SkyCoord` object.
@@ -213,9 +218,11 @@ def get_skycoord(targets):
     coords = [getattr(target, 'coord', target) for target in targets]
 
     # are all SkyCoordinate's in equivalent frames? If not, convert to ICRS
-    convert_to_icrs = not all([coord.frame.is_equivalent_frame(coords[0].frame) for coord in coords[1:]])
+    convert_to_icrs = not all(
+        [coord.frame.is_equivalent_frame(coords[0].frame) for coord in coords[1:]])
 
-    # we also need to be careful about handling mixtures of UnitSphericalRepresentations and others
+    # we also need to be careful about handling mixtures of
+    # UnitSphericalRepresentations and others
     targets_is_unitsphericalrep = [x.data.__class__ is
                                    UnitSphericalRepresentation for x in coords]
 
@@ -234,8 +241,15 @@ def get_skycoord(targets):
         frame = ICRS()
     else:
         # all the same frame, get the longitude and latitude names
-        lon_name, lat_name = [mapping.framename for mapping in
-                              coords[0].frame_specific_representation_info['spherical']]
+        try:
+            # from astropy v2.0, keys are classes
+            lon_name, lat_name = [
+                mapping.framename for mapping in
+                coords[0].frame_specific_representation_info[UnitSphericalRepresentation]]
+        except BaseException:            # whereas prior to that they were strings.
+            lon_name, lat_name = [mapping.framename for mapping in
+                                  coords[0].frame_specific_representation_info['spherical']]
+
         frame = coords[0].frame
         for coordinate in coords:
             longitudes.append(getattr(coordinate, lon_name))
@@ -258,4 +272,26 @@ def get_skycoord(targets):
         Instead, let's assign large distances to those objects with none.
         """
         distances = [distance if distance != 1 else 100*u.kpc for distance in distances]
-        return SkyCoord(longitudes, latitudes, distances, frame=frame)
\ No newline at end of file
+        return SkyCoord(longitudes, latitudes, distances, frame=frame)
+
+
+class SpecialObjectFlag(object):
+    """
+    Flag this object as a special non-fixed target, which has a ``get_*`` method
+    within astropy (like the Sun or Moon)
+    """
+    pass
+
+
+class SunFlag(SpecialObjectFlag):
+    """
+    Flag for a computation with the Sun
+    """
+    approx_sidereal_drift = 5 * u.min
+
+
+class MoonFlag(SpecialObjectFlag):
+    """
+    Flag for a computation with the Moon
+    """
+    approx_sidereal_drift = 60 * u.min
