@@ -7,16 +7,19 @@ import astropy.units as u
 from astropy.time import Time
 from astropy.coordinates import SkyCoord, get_sun, get_moon
 from astropy.utils import minversion
-from astropy.tests.helper import pytest
+import pytest
 
 from ..observer import Observer
-from ..target import FixedTarget
+from ..target import FixedTarget, get_skycoord
 from ..constraints import (AltitudeConstraint, AirmassConstraint, AtNightConstraint,
                            is_observable, is_always_observable, observability_table,
                            time_grid_from_range, SunSeparationConstraint,
                            MoonSeparationConstraint, MoonIlluminationConstraint,
                            TimeConstraint, LocalTimeConstraint, months_observable,
-                           max_best_rescale, min_best_rescale)
+                           max_best_rescale, min_best_rescale, PhaseConstraint,
+                           PrimaryEclipseConstraint, SecondaryEclipseConstraint,
+                           is_event_observable)
+from ..periodic import EclipsingSystem
 
 APY_LT104 = not minversion('astropy', '1.0.4')
 
@@ -35,8 +38,7 @@ def test_at_night_basic():
     targets = [vega, rigel, polaris]
     for time_range in time_ranges:
         # Calculate constraint using methods on astroplan.Observer:
-        observer_is_night = [subaru.is_night(time)
-                             for time in time_grid_from_range(time_range)]
+        observer_is_night = subaru.is_night(time_grid_from_range(time_range))
         observer_is_night_any = any(observer_is_night)
         observer_is_night_all = all(observer_is_night)
 
@@ -98,10 +100,8 @@ def test_compare_altitude_constraint_and_observer():
                                 for target in targets]
         # Check if each target meets altitude constraints using
         # is_always_observable and AltitudeConstraint
-        always_from_constraint = is_always_observable(AltitudeConstraint(min_alt,
-                                                                         max_alt),
-                                                      subaru, targets,
-                                                      time_range=time_range)
+        always_from_constraint = is_always_observable(AltitudeConstraint(
+            min_alt, max_alt), subaru, targets, time_range=time_range)
         assert all(always_from_observer == always_from_constraint)
 
 
@@ -121,9 +121,8 @@ def test_compare_airmass_constraint_and_observer():
                                 for target in targets]
         # Check if each target meets altitude constraints using
         # is_always_observable and AirmassConstraint
-        always_from_constraint = is_always_observable(AirmassConstraint(max_airmass),
-                                                      subaru, targets,
-                                                      time_range=time_range)
+        always_from_constraint = is_always_observable(
+            AirmassConstraint(max_airmass), subaru, targets, time_range=time_range)
         assert all(always_from_observer == always_from_constraint)
 
 
@@ -140,17 +139,17 @@ def test_sun_separation():
     constraint = SunSeparationConstraint(min=2*u.deg, max=10*u.deg)
     is_constraint_met = constraint(apo, [one_deg_away, five_deg_away,
                                          twenty_deg_away], times=time)
-    assert np.all(is_constraint_met == [[False], [True], [False]])
+    assert np.all(is_constraint_met == [False, True, False])
 
     constraint = SunSeparationConstraint(max=10*u.deg)
     is_constraint_met = constraint(apo, [one_deg_away, five_deg_away,
                                          twenty_deg_away], times=time)
-    assert np.all(is_constraint_met == [[True], [True], [False]])
+    assert np.all(is_constraint_met == [True, True, False])
 
     constraint = SunSeparationConstraint(min=2*u.deg)
     is_constraint_met = constraint(apo, [one_deg_away, five_deg_away,
                                          twenty_deg_away], times=time)
-    assert np.all(is_constraint_met == [[False], [True], [True]])
+    assert np.all(is_constraint_met == [False, True, True])
 
 
 def test_moon_separation():
@@ -167,18 +166,18 @@ def test_moon_separation():
     constraint = MoonSeparationConstraint(min=2*u.deg, max=10*u.deg)
     is_constraint_met = constraint(apo, [one_deg_away, five_deg_away,
                                          twenty_deg_away], times=time)
-    print(is_constraint_met)
-    assert np.all(is_constraint_met == [[False], [True], [False]])
+
+    assert np.all(is_constraint_met == [False, True, False])
 
     constraint = MoonSeparationConstraint(max=10*u.deg)
     is_constraint_met = constraint(apo, [one_deg_away, five_deg_away,
                                          twenty_deg_away], times=time)
-    assert np.all(is_constraint_met == [[True], [True], [False]])
+    assert np.all(is_constraint_met == [True, True, False])
 
     constraint = MoonSeparationConstraint(min=2*u.deg)
     is_constraint_met = constraint(apo, [one_deg_away, five_deg_away,
                                          twenty_deg_away], times=time)
-    assert np.all(is_constraint_met == [[False], [True], [True]])
+    assert np.all(is_constraint_met == [False, True, True])
 
 
 def test_moon_illumination():
@@ -224,15 +223,15 @@ def test_local_time_constraint_utc():
     subaru = Observer.at_site("Subaru")
     constraint = LocalTimeConstraint(min=dt.time(23, 50), max=dt.time(4, 8))
     is_constraint_met = constraint(subaru, None, times=time)
-    assert is_constraint_met == [True]
+    assert is_constraint_met is np.bool_(True)
 
     constraint = LocalTimeConstraint(min=dt.time(0, 2), max=dt.time(4, 3))
     is_constraint_met = constraint(subaru, None, times=time)
-    assert is_constraint_met == [False]
+    assert is_constraint_met is np.bool_(False)
 
     constraint = LocalTimeConstraint(min=dt.time(3, 8), max=dt.time(5, 35))
     is_constraint_met = constraint(subaru, None, times=time)
-    assert is_constraint_met == [True]
+    assert is_constraint_met is np.bool_(True)
 
 
 def test_local_time_constraint_hawaii_tz():
@@ -241,15 +240,15 @@ def test_local_time_constraint_hawaii_tz():
     subaru = Observer.at_site("Subaru", timezone="US/Hawaii")
     constraint = LocalTimeConstraint(min=dt.time(23, 50), max=dt.time(4, 8))
     is_constraint_met = constraint(subaru, None, times=time)
-    assert is_constraint_met == [True]
+    assert is_constraint_met is np.bool_(True)
 
     constraint = LocalTimeConstraint(min=dt.time(0, 2), max=dt.time(4, 3))
     is_constraint_met = constraint(subaru, None, times=time)
-    assert is_constraint_met == [False]
+    assert is_constraint_met is np.bool_(False)
 
     constraint = LocalTimeConstraint(min=dt.time(3, 8), max=dt.time(5, 35))
     is_constraint_met = constraint(subaru, None, times=time)
-    assert is_constraint_met == [True]
+    assert is_constraint_met is np.bool_(True)
 
 
 def test_docs_example():
@@ -278,12 +277,12 @@ def test_docs_example():
                for name, ra, dec in target_table]
 
     from astroplan import Constraint, is_observable
-    from astropy.coordinates import Angle
 
     class VegaSeparationConstraint(Constraint):
         """
         Constraint the separation from Vega
         """
+
         def __init__(self, min=None, max=None):
             """
             min : `~astropy.units.Quantity` or `None` (optional)
@@ -293,40 +292,20 @@ def test_docs_example():
                 Minimum acceptable separation between Vega and target. `None`
                 indicates no limit.
             """
-            self.min = min
-            self.max = max
+            self.min = min if min is not None else 0*u.deg
+            self.max = max if max is not None else 180*u.deg
 
         def compute_constraint(self, times, observer, targets):
-
-            # Vega's coordinate must be non-scalar for the dimensions
-            # to work out properly when combined with other constraints which
-            # test multiple times
-            vega = SkyCoord(ra=[279.23473479]*u.deg, dec=[38.78368896]*u.deg)
+            vega = SkyCoord(ra=279.23473479*u.deg, dec=38.78368896*u.deg)
 
             # Calculate separation between target and vega
-            vega_separation = Angle([vega.separation(target.coord)
-                                     for target in targets])
-
-            # If a maximum is specified but no minimum
-            if self.min is None and self.max is not None:
-                mask = vega_separation < self.max
-
-            # If a minimum is specified but no maximum
-            elif self.max is None and self.min is not None:
-                mask = self.min < vega_separation
-
-            # If both a minimum and a maximum are specified
-            elif self.min is not None and self.max is not None:
-                mask = ((self.min < vega_separation) & (vega_separation < self.max))
-
-            # Otherwise, raise an error
-            else:
-                raise ValueError("No max and/or min specified in "
-                                 "VegaSeparationConstraint.")
+            # Targets are automatically converted to SkyCoord objects
+            # by __call__ before compute_constraint is called.
+            vega_separation = vega.separation(targets)
 
             # Return an array that is True where the target is observable and
             # False where it is not
-            return mask
+            return (self.min < vega_separation) & (vega_separation < self.max)
 
     constraints = [VegaSeparationConstraint(min=5*u.deg, max=30*u.deg)]
     observability = is_observable(constraints, subaru, targets,
@@ -380,6 +359,7 @@ def test_rescale_minmax():
     rescaled[4] = (max_best_rescale(a, 0, 1, greater_than_max=0))[0]
     assert all(np.array([0.8, 0.2, 1, 0, 0]) == rescaled)
 
+
 constraint_tests = [
     AltitudeConstraint(),
     AirmassConstraint(2),
@@ -394,11 +374,87 @@ constraint_tests = [
 @pytest.mark.parametrize('constraint', constraint_tests)
 def test_regression_shapes(constraint):
     times = Time(["2015-08-28 03:30", "2015-09-05 10:30", "2015-09-15 18:35"])
-    targets = [FixedTarget(SkyCoord(350.7*u.deg, 18.4*u.deg)),
-               FixedTarget(SkyCoord(260.7*u.deg, 22.4*u.deg))]
+    targets = get_skycoord([FixedTarget(SkyCoord(350.7*u.deg, 18.4*u.deg)),
+                           FixedTarget(SkyCoord(260.7*u.deg, 22.4*u.deg))])
     lapalma = Observer.at_site('lapalma')
 
-    assert constraint(lapalma, targets, times).shape == (2, 3)
-    assert constraint(lapalma, [targets[0]], times).shape == (1, 3)
-    assert constraint(lapalma, [targets[0]], times[0]).shape == (1, 1)
-    assert constraint(lapalma, targets, times[0]).shape == (2, 1)
+    assert constraint(lapalma, targets[:, np.newaxis], times).shape == (2, 3)
+    assert constraint(lapalma, targets[0], times).shape == (3,)
+    assert np.array(constraint(lapalma, targets[0], times[0])).shape == ()
+    assert np.array(constraint(lapalma, targets, times[0])).shape == (2,)
+    with pytest.raises(ValueError):
+        constraint(lapalma, targets, times)
+
+
+def test_caches_shapes():
+    times = Time([2457884.43350526, 2457884.5029497, 2457884.57239415], format='jd')
+    m31 = SkyCoord(10.6847929*u.deg, 41.269065*u.deg)
+    ippeg = SkyCoord(350.785625*u.deg, 18.416472*u.deg)
+    htcas = SkyCoord(17.5566667*u.deg, 60.0752778*u.deg)
+    targets = get_skycoord([m31, ippeg, htcas])
+    observer = Observer.at_site('lapalma')
+    ac = AltitudeConstraint(min=30*u.deg)
+    assert ac(observer, targets, times, grid_times_targets=True).shape == (3, 3)
+    assert ac(observer, targets, times, grid_times_targets=False).shape == (3,)
+
+
+def test_eclipses():
+    subaru = Observer.at_site("Subaru")
+
+    epoch = Time('2016-01-01')
+    period = 3 * u.day
+    duration = 1 * u.hour
+    eclipsing_system = EclipsingSystem(primary_eclipse_time=epoch, orbital_period=period,
+                                       duration=duration, name='test system')
+    pec = PrimaryEclipseConstraint(eclipsing_system)
+    times = Time(['2016-01-01 00:00', '2016-01-01 03:00', '2016-01-02 12:00'])
+    assert np.all(np.array([True, False, False]) == pec(subaru, None, times))
+
+    sec = SecondaryEclipseConstraint(eclipsing_system)
+    times = Time(['2016-01-01 00:00', '2016-01-01 03:00', '2016-01-02 12:00'])
+    assert np.all(np.array([False, False, True]) == sec(subaru, None, times))
+
+    pc = PhaseConstraint(eclipsing_system, min=0.2, max=0.5)
+    times = Time(['2016-01-01 00:00', '2016-01-02 12:00', '2016-01-02 14:00'])
+    assert np.all(np.array([False, True, False]) == pc(subaru, None, times))
+
+
+def test_event_observable():
+
+    epoch = Time(2452826.628514, format='jd')
+    period = 3.52474859 * u.day
+    duration = 0.1277 * u.day
+
+    hd209458 = EclipsingSystem(primary_eclipse_time=epoch, orbital_period=period, duration=duration,
+                               name='HD 209458 b')
+    observing_time = Time('2017-09-15 10:20')
+
+    apo = Observer.at_site('APO')
+
+    target = FixedTarget.from_name("HD 209458")
+    n_transits = 100  # This is the roughly number of transits per year
+    ing_egr = hd209458.next_primary_ingress_egress_time(observing_time,
+                                                        n_eclipses=n_transits)
+    constraints = [AltitudeConstraint(min=0*u.deg), AtNightConstraint()]
+    observable = is_event_observable(constraints, apo, target,
+                                     times_ingress_egress=ing_egr)
+
+    # This answer was validated against the Czech Exoplanet Transit Database
+    # transit prediction service, at:
+    # http://var2.astro.cz/ETD/predict_detail.php?delka=254.1797222222222&submit=submit&sirka=32.780277777777776&STARNAME=HD209458&PLANET=b
+    # There is some disagreement, as the ETD considers some transits which begin
+    # before sunset or after sunrise to be observable.
+    cetd_answer = [[False, False, False, True, False, True, False, True, False,
+                    True, False, True, False, False, False, False, False, False,
+                    False, False, False, False, True, False, True, False, False,
+                    False, False, False, False, False, False, False, False, False,
+                    False, False, False, False, False, False, False, False, False,
+                    False, False, False, False, False, False, False, False, False,
+                    False, False, False, True, False, False, False, False, False,
+                    False, False, False, False, False, False, False, False, False,
+                    True, False, True, False, False, False, False, False, False,
+                    False, False, False, False, False, False, True, False, True,
+                    False, True, False, True, False, True, False, True, False,
+                    False]]
+
+    assert np.all(observable == np.array(cetd_answer))
